'' This demo tests TLC5940 board from Brilldea (www.brilldea.com)
''
'' Uses a different driver than supplied
''
'' New driver uses 1 COG rather than 2 (Counters used for gsclk)
'' Supports 4096 light intensities per color (rather than 256)
'' Supports status from tlc5940 using SOUT pin if connected
''
'' For 1 board, jumper SRTN-OUT and SOUT-BLU
''
'' Connect 2x5way header to prop board
''  with
''       VCC     - +3.3V         GND     - Vss         
''       XLAT    - P20           SCLK    - P24                                                           
''       SIN     - P21           SRTN    - P25 or not connected               
''       GSCLK   - P22           BLNK    - P26 - also connect a 10K pull-up resistor to +3.3V
''       DCPRG   - P23 (or Vss)  VPRG    - P27 (or Vss)                
''
con
  _clkmode        = xtal1 + pll16x                '
  _xinfreq        = 5_000_000                     '

  NUM_TEST_BOARDS = 1

obj
  uarts         : "pcFullDuplexSerial4FC"               '1 COG for 4 serial ports

  leds          : "tlc5940gsclk"                        '1 COG
  
  config        : "config"                              'no COG

var
  long  LODstatus[LEDS#SIZE_LODSTATUS_LONGS]
  byte  TEFstatus[LEDS#SIZE_TEFSTATUS_BYTES]
  long  DCstatus[LEDS#SIZE_DCSTATUS_LONGS]
  long  LODstatusOld[LEDS#SIZE_LODSTATUS_LONGS]         'old status to see if changed
  byte  TEFstatusOld[LEDS#SIZE_TEFSTATUS_BYTES]         'old status to see if changed
        
pub Main | board,channel,red,blue,green
  config.Init(@configinfo)                              'config table for this test
  waitcnt(clkfreq*3 + cnt)

  longfill(@board,0,5)                                  'Dont really need this but useful when trying different
                                                        'loops that may not initialize all colors
  longfill(@LODStatus,0,LEDS#SIZE_LODSTATUS_LONGS)      'zero all the status arrays
  bytefill(@TEFStatus,0,LEDS#SIZE_TEFSTATUS_BYTES)
  longfill(@DCstatus,0,LEDS#SIZE_DCSTATUS_LONGS)
  longfill(@LODStatusOld,-1,LEDS#SIZE_LODSTATUS_LONGS)  '-1 all the old status arrays so always difference first time
  bytefill(@TEFStatusOld,-1,LEDS#SIZE_TEFSTATUS_BYTES)
  
  uarts.Init
  uarts.AddPort(0,config.GetPin(CONFIG#DEBUG_RX),config.GetPin(CONFIG#DEBUG_TX), {
}   UARTS#PINNOTUSED,UARTS#PINNOTUSED,UARTS#DEFAULTTHRESHOLD,                    {
}   UARTS#NOMODE,UARTS#BAUD115200)                      'Add debug port
  uarts.Start                                           'Start the ports
  
  uarts.str(0,string("Starting",13))

  leds.Start(NUM_TEST_BOARDS, {                         'Start the led driver
}   config.GetPin(CONFIG#TLC5940_GSCLK),config.GetPin(CONFIG#TLC5940_BLANK),{
}   config.GetPin(CONFIG#TLC5940_SCLK), config.GetPin(CONFIG#TLC5940_SIN),  {
}   config.GetPin(CONFIG#TLC5940_SOUT), config.GetPin(CONFIG#TLC5940_XLAT), {
}   config.GetPin(CONFIG#TLC5940_VPRG), config.GetPin(CONFIG#TLC5940_DCPRG))

  uarts.str(0,string("Running",13))
  waitcnt(clkfreq/10 + cnt)
  leds.UpdateScreen                                     'Force update so status is valid
   
  repeat
    repeat board from 0 to NUM_TEST_BOARDS-1
      repeat channel from 15 to 0
        PrintChannel(channel)
        repeat blue from 0 to 4095 step 256
          GetStatus                                 'Get status and print if it has changed
          if CompareStatus
            PrintStatus          
          repeat green from 0 to 4095 step 256
            repeat red from 0 to 4095 step 256
              leds.SetChValue(board, channel, red, green, blue)
              leds.UpdateScreen
              waitcnt(5000 + cnt)

PRI PrintChannel(ch)
  uarts.str(0,string("Channel: "))
  uarts.dec(0,ch)
  uarts.tx(0,13)

PRI GetStatus
  leds.GetLODStatus(@LODStatus)
  leds.GetTEFStatus(@TEFStatus)

PRI CompareStatus | i
''return 1 if status has changed and update old status to new status
  result := 0
  repeat i from 0 to LEDS#SIZE_LODSTATUS_LONGS-1
    if LODStatus[i] <> LODStatusOld[i]
      result := 1
      quit
  repeat i from 0 to LEDS#SIZE_TEFSTATUS_BYTES-1
    if TEFStatus[i] <> TEFStatusOld[i]
      result := 1
      quit
  if result == 1                                        'if status has changed update the old status
    longmove(@LODStatusOld,@LODStatus,LEDS#SIZE_LODSTATUS_LONGS)
    bytemove(@TEFStatusOld,@TEFStatus,LEDS#SIZE_TEFSTATUS_BYTES)
  
PRI PrintStatus | space
  space := string(" ")          
  uarts.str(0,string("Status: "))           
  uarts.hex(0,LODStatus[0],4)
  uarts.str(0,string(" "))
  uarts.hex(0,LODStatus[1],4)
  uarts.str(0,string(" "))
  uarts.hex(0,LODStatus[2],4)
  uarts.str(0,string(" "))
  uarts.hex(0,TEFStatus[0],1)
  uarts.str(0,string(" "))
  uarts.hex(0,TEFStatus[1],1)
  uarts.str(0,string(" "))
  uarts.hex(0,TEFStatus[2],1)
  uarts.tx(0,13)

DAT
'' Pin configuration table for this application
''
configinfo    word CONFIG#NOT_USED              'pin 0
              word CONFIG#NOT_USED              'pin 1
              word CONFIG#NOT_USED              'pin 2
              word CONFIG#NOT_USED              'pin 3
              word CONFIG#NOT_USED              'pin 4
              word CONFIG#NOT_USED              'pin 5
              word CONFIG#NOT_USED              'pin 6
              word CONFIG#NOT_USED              'pin 7
              word CONFIG#NOT_USED              'pin 8
              word CONFIG#NOT_USED              'pin 9
              word CONFIG#NOT_USED              'pin 10
              word CONFIG#NOT_USED              'pin 11
              word CONFIG#NOT_USED              'pin 12
              word CONFIG#NOT_USED              'pin 13
              word CONFIG#NOT_USED              'pin 14
              word CONFIG#NOT_USED              'pin 15
              word CONFIG#NOT_USED              'pin 16
              word CONFIG#NOT_USED              'pin 17
              word CONFIG#NOT_USED              'pin 18
              word CONFIG#NOT_USED              'pin 19
              word CONFIG#TLC5940_XLAT          'pin 20
              word CONFIG#TLC5940_SIN           'pin 21
              word CONFIG#TLC5940_GSCLK         'pin 22
              word CONFIG#TLC5940_DCPRG         'pin 23
              word CONFIG#TLC5940_SCLK          'pin 24
              word CONFIG#TLC5940_SOUT          'pin 25
              word CONFIG#TLC5940_BLANK         'pin 26
              word CONFIG#TLC5940_VPRG          'pin 27
              word CONFIG#I2C_SCL1              'pin 28
              word CONFIG#I2C_SDA1              'pin 29
              word CONFIG#DEBUG_TX              'pin 30
              word CONFIG#DEBUG_RX              'pin 31           